@@ -1,6 +1,11 @@
 'use strict';
 
-var NQPInt = require('./nqp-int.js');
+const coercions = require('./coercions.js');
+
+const NQPInt = require('./nqp-int.js');
+
+const JSBI = require('jsbi');
+const asIntN = require('./as-int-n.js');
 
 class CodePair {
   constructor(STable) {
@@ -15,16 +20,16 @@ class CodePair {
   }
 
   setupSTable() {
-    var fetch = this.fetch;
-    var store = this.store;
+    const fetch = this.fetch;
+    const store = this.store;
 
     this.STable.addInternalMethods(class {
       $$assignunchecked(ctx, value) {
-        store.$$call(ctx, {}, this, value);
+        return store.$$call(ctx, {}, this, value);
       }
 
       $$assign(ctx, value) {
-        store.$$call(ctx, {}, this, value);
+        return store.$$call(ctx, {}, this, value);
       }
 
       $$iscont() {
@@ -73,8 +78,8 @@ class NativeRef {
   }
 
   setupSTable() {
-    var primitiveType = this.STable.REPR.primitiveType;
-    var STable = this.STable;
+    const primitiveType = this.STable.REPR.primitiveType;
+    const STable = this.STable;
 
     /* TODO - take the hll in account when converting to object */
 
@@ -100,23 +105,35 @@ class NativeRef {
           this.set(value);
         }
 
-        $$decont_i(ctx, value) {
+        $$decont_i(ctx) {
           return this.get();
         }
 
-        $$decont(ctx, value) {
-          var hll = STable.hllOwner;
+        $$getInt() {
+          return this.get();
+        }
+
+        $$getNum() {
+          return this.get();
+        }
+
+        $$getStr() {
+          return this.get().toString();
+        }
+
+        $$decont(ctx) {
+          let hll = STable.hllOwner;
           if (hll === undefined) {
             hll = ctx.codeRef().staticCode.hll;
           }
 
-          var type = hll.get('int_box');
+          const type = hll.get('int_box');
           if (!type) {
             // HACK - nqp still uses NQPInt instead of the thing in the hll config
             return new NQPInt(this.get());
           } else {
-            var repr = type._STable.REPR;
-            var obj = repr.allocate(type._STable);
+            const repr = type.$$STable.REPR;
+            const obj = repr.allocate(type.$$STable);
             obj.$$setInt(this.get());
             return obj;
           }
@@ -140,23 +157,35 @@ class NativeRef {
           this.set(value);
         }
 
-        $$decont_n(ctx, value) {
+        $$decont_n(ctx) {
           return this.get();
         }
 
-        $$decont(ctx, value) {
-          var hll = STable.hllOwner;
+        $$getInt() {
+          return this.get()|0;
+        }
+
+        $$getNum() {
+          return this.get();
+        }
+
+        $$getStr() {
+          return coercions.numToStr(this.get());
+        }
+
+        $$decont(ctx) {
+          let hll = STable.hllOwner;
           if (hll === undefined) {
             hll = ctx.codeRef().staticCode.hll;
           }
 
-          var type = hll.get('num_box');
+          const type = hll.get('num_box');
           if (!type) {
             // HACK - nqp still uses raw javascript numbers instead of the thing in the hll config
             return this.get();
           } else {
-            var repr = type._STable.REPR;
-            var obj = repr.allocate(type._STable);
+            const repr = type.$$STable.REPR;
+            const obj = repr.allocate(type.$$STable);
             obj.$$setNum(this.get());
             return obj;
           }
@@ -184,23 +213,35 @@ class NativeRef {
           this.set(value);
         }
 
-        $$decont_s(ctx, value) {
+        $$decont_s(ctx) {
           return this.get();
         }
 
-        $$decont(ctx, value) {
-          var hll = STable.hllOwner;
+        $$getStr() {
+          return this.get();
+        }
+
+        $$getInt() {
+          return coercions.strToInt(this.get());
+        }
+
+        $$getNum() {
+          return coercions.strToNum(this.get());
+        }
+
+        $$decont(ctx) {
+          let hll = STable.hllOwner;
           if (hll === undefined) {
             hll = ctx.codeRef().staticCode.hll;
           }
 
-          var type = hll.get('str_box');
+          const type = hll.get('str_box');
           if (!type) {
             // HACK - nqp still uses raw javascript strings instead of the thing in the hll config
             return this.get();
           } else {
-            var repr = type._STable.REPR;
-            var obj = repr.allocate(type._STable);
+            const repr = type.$$STable.REPR;
+            const obj = repr.allocate(type.$$STable);
             obj.$$setStr(this.get());
             return obj;
           }
@@ -210,6 +251,68 @@ class NativeRef {
           this.set(value.$$getStr());
         }
       });
+    } else if (primitiveType == 4) { // int64
+      this.STable.addInternalMethods(class {
+        $$iscont_i() {
+          return 1;
+        }
+
+        $$iscont() {
+          return 1;
+        }
+
+        $$isrwcont() {
+          return 1;
+        }
+
+        $$assign_i64(ctx, value) {
+          this.set(value);
+        }
+
+        $$assign_u64(ctx, value) {
+          this.set(value);
+        }
+
+        $$assign(ctx, value) {
+          this.set(value.$$getInt64());
+        }
+
+        $$getStr() {
+          return this.get().toString();
+        }
+
+        $$getInt64() {
+          return this.get();
+        }
+
+        // HACK till we get better int64 support in rakudo.js
+        $$getInt() {
+          return JSBI.toNumber(asIntN.asIntN(32, this.get()));
+        }
+
+        // HACK till we get better int64 support in rakudo.js
+        $$assign_i(ctx, value) {
+          this.set(JSBI.BigInt(value));
+        }
+
+        $$getUint64() {
+          return asIntN.asUintN(64, this.get());
+        }
+
+        $$decont(ctx) {
+          let hll = STable.hllOwner;
+          if (hll === undefined) {
+            hll = ctx.codeRef().staticCode.hll;
+          }
+
+          const type = hll.get('int_box');
+
+          const repr = type.$$STable.REPR;
+          const obj = repr.allocate(type.$$STable);
+          obj.$$setInt64(this.get());
+          return obj;
+        }
+      });
     } else {
       throw 'incorrect type of NativeRef: ' + primitiveType;
     }
