@@ -526,10 +526,12 @@ function wrapBuffer(buffer, type) {
   return stringified;
 }
 
-op.spawnprocasync = /*async*/ function(ctx, queue, args, cwd, env, config) {
+op.spawnprocasync = /*async*/ function(ctx, queue, prog, args, cwd, env, config) {
+  const stringified = /*await*/ stringifyArray(ctx, args);
   const options = {
     shell: false,
     windowsVerbatimArguments: true,
+    argv0: stringified.shift(),
     cwd: cwd,
     env: /*await*/ stringifyEnv(ctx, env),
     stdio: [
@@ -539,9 +541,7 @@ op.spawnprocasync = /*async*/ function(ctx, queue, args, cwd, env, config) {
     ],
   };
 
-  const stringified = /*await*/ stringifyArray(ctx, args);
-
-  const result = child_process.spawnSync(stringified.shift(), stringified, options);
+  const result = child_process.spawnSync(prog, stringified, options);
 
   if (config.content.get('ready')) {
     /*await*/ config.content.get('ready').$$call(ctx, null);
